/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"

cvar_t		vid_brightness	= {"vid_brightness",	"1",	true};
cvar_t		vid_contrast	= {"vid_contrast",		"1",	true};
cvar_t		vid_gamma		= {"vid_gamma",			"1",	true};

#if defined VIDMODEXT_GAMMA_RAMP || defined _WIN32
float		brightness		= -1;
float		contrast		= -1;
float		gamma			= -1;
qboolean	hwgamma			= false;

int VID_SetGamma (float gamma, float scale, float base);

void VID_UpdateGamma (qboolean force)
{
	if (cls.state == ca_dedicated)
		return;

	if (vid_brightness.fvalue	== brightness	&&
		vid_contrast.fvalue		== contrast		&&
		vid_gamma.fvalue		== gamma		&&
		!force)
		return;

	if (vid_brightness.fvalue < 1.0)
		Cvar_SetValue("vid_brightness", 1.0);
	if (vid_brightness.fvalue > 5.0)
		Cvar_SetValue("vid_brightness", 5.0);

	if (vid_contrast.fvalue < 0.2)
		Cvar_SetValue("vid_contrast", 0.2f);
	if (vid_contrast.fvalue > 1)
		Cvar_SetValue("vid_contrast", 1);

	if (vid_gamma.fvalue < 0.1)
		Cvar_SetValue("vid_gamma", 0.1f);
	if (vid_gamma.fvalue > 5.0)
		Cvar_SetValue("vid_gamma", 5.0);

	brightness	= vid_brightness.fvalue;
	contrast	= vid_contrast.fvalue;
	gamma		= vid_gamma.fvalue;

	hwgamma		= VID_SetGamma (gamma, brightness * contrast, 1 - contrast);

	if (!hwgamma)
		Con_Printf ("Hardware gamma not supported.\n");
}

void BuildGammaTable (float gamma, float scale, float base, unsigned short *out)
{
	int		i, adjusted;
	double	invgamma, d;

	gamma = bound (0.1, gamma, 5.0);

	if (gamma == 1) // LordHavoc: dodge the math
	{
		for (i=0; i<256; i++)
		{
			adjusted	= (int) (i * 256.0 * scale + base * 65535.0);
			out[i]		= bound (0, adjusted, 65535);
		}
	}
	else
	{
		invgamma = 1.0 / gamma;

		for (i=0; i<256; i++)
		{
			d			= MR_pow((double) i * 0.004f, invgamma) * scale + base;
			adjusted	= (int) (65535.0 * d);
			out[i]		= bound (0, adjusted, 65535);
		}
	}
}

void Gamma_Init (void)
{
	Cvar_RegisterVariable(&vid_brightness);
	Cvar_RegisterVariable(&vid_contrast);
	Cvar_RegisterVariable(&vid_gamma);
}
#endif

