/*
Copyright (C) 2001-2002 Tomas Jakobsson.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"
#include	"../menu.h"

// #warning this should not be hardcoded...
map_t	maps[32];
int	nummaps;

void LoadRecords (void)
{
	FILE	*recordfile;
	int		i, j;
	list_t mapnames[32];

	// we need to know which maps are available
	// maps0.lst
	nummaps = LoadList("maps/maps0.lst", mapnames, true);
	if(nummaps > 32)
	    nummaps = 32;

	for(i=0;i<nummaps;i++)
	{
	    _snprintf (maps[i].name, sizeof(maps[i].name), mapnames[i].name);
	    _snprintf (maps[i].description, sizeof(maps[i].description), mapnames[i].name);
	}

	// maps1.lst
	nummaps += LoadList("maps/maps1.lst", mapnames, false);
	if(nummaps > 32)
	    nummaps = 32;

	for(j=0;i<nummaps;i++,j++)
	{
	    _snprintf (maps[i].name, sizeof(maps[i].name), mapnames[j].name);
	    _snprintf (maps[i].description, sizeof(maps[i].description), mapnames[j].name);
	}

	// maps2.lst
	nummaps += LoadList("maps/maps2.lst", mapnames, false);
	if(nummaps > 32)
	    nummaps = 32;

	for(j=0;i<nummaps;i++,j++)
	{
	    _snprintf (maps[i].name, sizeof(maps[i].name), mapnames[j].name);
	    _snprintf (maps[i].description, sizeof(maps[i].description), mapnames[j].name);
	}

	Con_Printf("Number of maps: %d\n", nummaps);

	recordfile = fopen("data/records.txt","r");

	if (recordfile)
	{
		for (i=0; i<nummaps; i++)
		{
			fscanf (recordfile, "%s", maps[i].lapholder);
			fscanf (recordfile, "%s", maps[i].raceholder);
			fscanf (recordfile, "%f", &maps[i].laprecord);
			fscanf (recordfile, "%f", &maps[i].racerecord);

			// value was not readable? setting default value for entry
			if(maps[i].laprecord == 0.0 || maps[i].racerecord == 0.0)
			{
				strcpy(maps[i].lapholder, "*****");
				strcpy(maps[i].raceholder, "*****");	
				maps[i].laprecord = 300;
				maps[i].racerecord = 1500;
			}
		}

		Con_Printf("Loaded records\n");

		fclose (recordfile);
	}
	else
	{
		Con_Printf("Failed to load records\n");

		for (i=0; i<nummaps; i++)
		{
			_snprintf (maps[i].lapholder, sizeof(maps[i].lapholder), "*****");
			_snprintf (maps[i].raceholder, sizeof(maps[i].raceholder), "*****");
			maps[i].laprecord	= 300;
			maps[i].racerecord	= 1500;
		}
	}
}

void SaveRecords (void)
{
	FILE	*recordfile;
	int		i;

	recordfile = fopen("data/records.txt","w");

	if (recordfile)
	{
		for (i=0; i<nummaps; i++)
		{
			fprintf (recordfile, "%s\n", maps[i].lapholder);
			fprintf (recordfile, "%s\n", maps[i].raceholder);
			fprintf (recordfile, "%f\n", maps[i].laprecord);
			fprintf (recordfile, "%f\n", maps[i].racerecord);
		}

		fprintf (recordfile, "// Record file made by MiniRacer.\n");
		fprintf (recordfile, "// This file can be edited easily.\n");
		fprintf (recordfile, "// But do you really wanna cheat?\n");

		Con_Printf("Saved records\n");

		fclose(recordfile);
	}
}
