/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include	"../quakedef.h"
#include	"../menu.h"
#include	"../vid.h"

scrdest_t	scr_dest = scr_menu;

qboolean	scr_drawconsole;
qboolean	scr_initialized;
qboolean	scr_disabled_for_loading;

float		scr_disabled_time;

void	SCR_ScreenShot_f (void);
void	GL_Set2D		 (void);
extern	qboolean	scr_drawmenu;

//=============================================================================

/*
======================
Draw_ConsoleBackground
======================
*/
void SCR_DrawLogo (void)
{
	extern	int	logo_texture;

	GL_Set2D ();

	glBindTexture (GL_TEXTURE_2D, logo_texture);

	glBegin (GL_QUADS);
		glTexCoord2f (0, 0);	glVertex2f (0,			0);
		glTexCoord2f (1, 0);	glVertex2f (vid.width,	0);
		glTexCoord2f (1, 1);	glVertex2f (vid.width,	vid.height);
		glTexCoord2f (0, 1);	glVertex2f (0,			vid.height);
	glEnd ();

	GL_EndRendering ();

#ifdef _WIN32
	Sleep (3000);
#else
	sleep (3);
#endif
}

/*
========
SCR_Init	
========
*/
void SCR_Init (void)
{
	Cmd_AddCommand ("screenshot",SCR_ScreenShot_f);

	scr_initialized = true;

	SCR_DrawLogo ();
}

/*
=================
SCR_DrawCamValues
=================
*/
void SCR_DrawCamValues (void)
{
	extern	cvar_t		cam_forward;
	extern	cvar_t		cam_right;
	extern	cvar_t		cam_up;
	extern	cvar_t		cam_pitch;
	extern	cvar_t		cam_roll;
	extern	cvar_t		cam_yaw;
	extern	qboolean	draw_camvalues;

	if (!draw_camvalues)
		return;

	Draw_String (0, 64,  va("Forward %3.0i", cam_forward.ivalue),	0, 255, 255, 255, 255);
	Draw_String (0, 80,  va("Right   %3.0i", cam_right.ivalue),		0, 255, 255, 255, 255);
	Draw_String (0, 96,  va("Up      %3.0i", cam_up.ivalue),		0, 255, 255, 255, 255);
	Draw_String (0, 112, va("Pitch   %3.0i", cam_pitch.ivalue),		0, 255, 255, 255, 255);
	Draw_String (0, 128, va("Yaw     %3.0i", cam_yaw.ivalue),		0, 255, 255, 255, 255);
	Draw_String (0, 144, va("Roll    %3.0i", cam_roll.ivalue),		0, 255, 255, 255, 255);
}

/*
===========
SCR_DrawFPS
===========
*/
void SCR_DrawFPS (void)
{
	static double	currtime, frametimes[32];
	double			newtime, total=0;
	static int		framecycle=0;
	int				calc, count=0, i;
	char			temp[32];

	newtime	= Sys_FloatTime();
	frametimes[framecycle] = newtime - currtime;

	while (total < 0.2 && count < 32 && frametimes[i = (framecycle - count) & 31])
	{
		total += frametimes[i];
		count++;
	}

	framecycle++;
	framecycle &= 31;

	calc = (int) (((double) count / total) + 0.5);

	currtime	= newtime;

	_snprintf(temp, sizeof(temp), "%3i", calc);

	Draw_String (vid.width - 36, 0, temp, 0, 255, 255, 255, 255);
}

void S_Play_2DSound (byte wavnum, int volume);
void S_Stop_Sound (byte wavnum);

extern	cvar_t	volume2;
extern	cvar_t	volume3;

/*
=============
SCR_DrawCount
=============
*/
void SCR_DrawCount (void)
{
	static	float		fade_time;
	static	qboolean	boop3;
	static	qboolean	boop2;
	static	qboolean	boop1;
	static	qboolean	beep;
	
	int					x1, y1 ,x2 ,y2;
	float				fade;

	extern int			countdown_texture[4];

	if(!countdown.used)
		return;

	if((cl.time - countdown.time) < 1)
	{
		if(!boop3)
		{
			boop3 = true;
			fade_time = cl.time;
			S_Play_2DSound (2, volume2.ivalue);
		}
		glBindTexture(GL_TEXTURE_2D, countdown_texture[3]);
	}
	else if((cl.time - countdown.time) < 2)
	{
		if(!boop2)
		{
			boop2 = true;
			fade_time = cl.time;
			S_Play_2DSound (2, volume2.ivalue);
		}
		glBindTexture(GL_TEXTURE_2D, countdown_texture[2]);
	}
	else if((cl.time - countdown.time) < 3)
	{
		if(!boop1)
		{
			boop1 = true;
			fade_time = cl.time;
			S_Play_2DSound (2, volume2.ivalue);
		}
		glBindTexture(GL_TEXTURE_2D, countdown_texture[1]);
	}
	else if((cl.time - countdown.time) < 4)
	{
		if(!beep)
		{
			beep = true;
			fade_time = cl.time;
			S_Play_2DSound (3, volume3.ivalue);
		}
		glBindTexture(GL_TEXTURE_2D, countdown_texture[0]);
	}
	else
	{
		countdown.time	= 0;
		countdown.used	= false;
		boop3			= false;
		boop2			= false;
		boop1			= false;
		beep			= false;

		return;
	}

	fade = cl.time - fade_time;

	if((cl.time - countdown.time) > 3)
	{
		x1 = - (192 * fade);
		y1 = - (64 * fade);

		x2 = + (192 * fade);
		y2 = + (64 * fade);
	}
	else
	{
		x1 = - (64 * fade);
		y1 = - (64 * fade);

		x2 = + (64 * fade);
		y2 = + (64 * fade);
	}

	glPushMatrix();

	glTranslatef (vid.width  * 0.5, vid.height * 0.5, 0);

	glColor4ub	(255,255,255, (byte)(255*(1 - fade)));
	glEnable	(GL_BLEND);
	glTexEnvi	(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	glBegin (GL_QUADS);
		glTexCoord2f (0, 0); glVertex2f (x1, y1);
		glTexCoord2f (1, 0); glVertex2f (x2, y1);
		glTexCoord2f (1, 1); glVertex2f (x2, y2);
		glTexCoord2f (0, 1); glVertex2f (x1, y2);
	glEnd ();

	glTexEnvi	(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
	glDisable	(GL_BLEND);
	glColor4ub	(255,255,255,255);

	glPopMatrix();
}

/*
======================
SCR_SetUpToDrawConsole
======================
*/
void SCR_SetUpToDrawConsole (void)
{
	Con_CheckResize ();
		
// decide on the height of the console
	con_forcedup = !cl.worldmodel || cls.signon != SIGNONS;

	if (con_forcedup || key_dest == key_console)
	{
		scr_dest = scr_console;
		scr_drawconsole = true;
	}
	else
		scr_drawconsole = false;
}

/* 
================
SCR_ScreenShot_f
================
*/  
void SCR_ScreenShot_f (void) 
{
	byte		*buffer;
	FILE		*tgafile;
	char		tganame[MAX_OSPATH]; 
	char		checkname[MAX_OSPATH];
	int			i, c, temp;

	Sys_mkdir ("data/screenies");	

	MR_strcpy (tganame,"Mini000.tga");
		
	for (i=0; i<100; i++) 
	{ 
		tganame[5] = i/10 + '0'; 
		tganame[6] = i%10 + '0'; 
		_snprintf (checkname, sizeof(checkname), "data/screenies/%s", tganame);
		if (Sys_FileTime (checkname) == -1)
			break;
	} 

	c = 18+vid.width*vid.height*3;

	buffer		= malloc (c);

	MR_memset (buffer, 0, 18);

	buffer[2]	= 2;
	buffer[12]	= vid.width&255;
	buffer[13]	= vid.width>>8;
	buffer[14]	= vid.height&255;
	buffer[15]	= vid.height>>8;
	buffer[16]	= 24;

	glReadPixels (0, 0, vid.width, vid.height, GL_RGB, GL_UNSIGNED_BYTE, buffer+18 ); 

	// swap rgb to bgr
	for (i=18 ; i<c ; i+=3)
	{
		temp		= buffer[i];
		buffer[i]	= buffer[i+2];
		buffer[i+2] = temp;
	}

	tgafile = fopen(checkname, "wb");

	fwrite(buffer, c, 1, tgafile);

	fclose(tgafile);

	free (buffer);
	Con_Printf ("Wrote %s\n", tganame);
} 

/*
======================
SCR_BeginLoadingPlaque
======================
*/
void SCR_BeginLoadingPlaque (void)
{
	if (cls.state != ca_connected)
		return;
	if (cls.signon != SIGNONS)
		return;
	
// redraw with no console and the loading plaque
	Con_ClearNotify ();
	scr_drawconsole				= false;

	scr_disabled_for_loading	= true;
	scr_disabled_time			= realtime;
}

/*
====================
SCR_EndLoadingPlaque
====================
*/
void SCR_EndLoadingPlaque (void)
{
	scr_disabled_for_loading = false;
	Con_ClearNotify ();
}

//=============================================================================

/*
================
SCR_UpdateScreen
================
*/

//extern	qboolean	intro_running;
extern	qboolean	model_visible;
extern	int			car_texture;

void DrawLaps (void);
void DrawTimes (void);
void GL_DrawDashBoard (void);
void View_RenderView (void);
void VID_UpdateGamma (qboolean force);
void Draw_CharArrays (void);
void R_SetupMenuModel (int skin);
void S_Stop_Music (void);
void S_Play_Music (int volume);

void SCR_UpdateScreen (void)
{
	extern cvar_t volume6;

	if (!scr_initialized || !con_initialized || scr_disabled_for_loading)
		return;				// not initialized yet

	GL_EndRendering ();
	
#ifdef VIDMODEXT_GAMMA_RAMP
	VID_UpdateGamma (false);	
#endif
	SCR_SetUpToDrawConsole ();

	if(!scr_drawconsole && !scr_drawmenu)
	{
		View_RenderView ();

		GL_Set2D ();

		DrawLaps ();
		DrawTimes ();

		SCR_DrawCount ();

		SCR_DrawCamValues ();
		GL_DrawDashBoard ();

		S_Stop_Music ();
	}
	else
	{
		S_Stop_Sound (4);
		S_Stop_Sound (5);
		S_Stop_Sound (6);
		S_Stop_Sound (7);

		GL_Set2D ();

		Menu_Mouse ();
	}

	SCR_DrawFPS ();

	if (scr_drawconsole && !scr_drawmenu)
		Con_DrawConsole ();
	else if (key_dest == key_game || key_dest == key_message)
		Con_DrawNotify ();

	SCR_DrawMenu ();

	if (scr_drawmenu || scr_drawconsole)
		S_Play_Music (volume6.ivalue);

	Draw_CharArrays ();

	if (model_visible)
		R_SetupMenuModel (car_texture);

	glFlush ();
}
