/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// gl_warp.c -- sky polygons

#include	"../quakedef.h"

void R_LoadSkyBox (void)
{
}

extern camera_t	camera;

#define R_SkyBoxPolyVec(s,t,x,y,z) \
	glTexCoord2f(s, t);\
	glVertex3f((x) + camera.origin[0], (y) + camera.origin[1], (z) + camera.origin[2]);

/*
============
R_DrawSkyBox
============
*/
void R_DrawSkyBox (void)
{
	extern int	sky_texture[5];

	glBindTexture(GL_TEXTURE_2D, sky_texture[0]); // front
	glBegin(GL_QUADS);
		R_SkyBoxPolyVec (0.998f, 0.002f,  2048, -2048,  2048);
		R_SkyBoxPolyVec (0.998f, 0.998f,  2048, -2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.998f,  2048,  2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.002f,  2048,  2048,  2048);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, sky_texture[1]); // back
	glBegin(GL_QUADS);
		R_SkyBoxPolyVec (0.998f, 0.002f, -2048,  2048,  2048);
		R_SkyBoxPolyVec (0.998f, 0.998f, -2048,  2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.998f, -2048, -2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.002f, -2048, -2048,  2048);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, sky_texture[2]); // right
	glBegin(GL_QUADS);
		R_SkyBoxPolyVec (0.998f, 0.002f,  2048,  2048,  2048);
		R_SkyBoxPolyVec (0.998f, 0.998f,  2048,  2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.998f, -2048,  2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.002f, -2048,  2048,  2048);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, sky_texture[3]); // left
	glBegin(GL_QUADS);
		R_SkyBoxPolyVec (0.998f, 0.002f, -2048, -2048,  2048);
		R_SkyBoxPolyVec (0.998f, 0.998f, -2048, -2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.998f,  2048, -2048, -2048);
		R_SkyBoxPolyVec (0.002f, 0.002f,  2048, -2048,  2048);
	glEnd();

	glBindTexture(GL_TEXTURE_2D, sky_texture[4]); // up
	glBegin(GL_QUADS);
		R_SkyBoxPolyVec (0.998f, 0.002f,  2048, -2048,  2048);
		R_SkyBoxPolyVec (0.998f, 0.998f,  2048,  2048,  2048);
		R_SkyBoxPolyVec (0.002f, 0.998f, -2048,  2048,  2048);
		R_SkyBoxPolyVec (0.002f, 0.002f, -2048, -2048,  2048);
	glEnd();
}

/*
===========
R_DrawWater
===========
*/
void R_DrawWater (msurface_t *waterchain)
{
	extern int	water_texture[32];
	msurface_t	*s;
	glpoly_t	*p;
	float		*v;
	int			i, j;
	byte		blue;

	extern cvar_t	_timeofday;

	switch (_timeofday.ivalue)
	{
	case 1:
		blue	= 48;
		break;

	case 2:
		blue	= 16;
		break;

	case 3:
		blue	= 48;
		break;

	default:
		blue	= 128;
		break;
	}

	j = (int)(host_time * 20) & 31;
	glBindTexture (GL_TEXTURE_2D, water_texture[j]);

	glTexEnvi	(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glColor4ub	(0, 0, blue, 255);

	for (s = waterchain; s; s=s->texturechain) 
	{
		for (p=s->polys ; p ; p=p->next)
		{
			glBegin (GL_POLYGON);
			for (i=0,v=p->verts[0] ; i<p->numverts ; i++, v+=VERTEXSIZE)
			{
				glTexCoord2f (v[3], v[4]);
				glVertex3fv (v);
			}
			glEnd ();
		}
	}

	glColor4ub	(255,255,255,255);
	glTexEnvi	(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
}

/*
==========
R_DrawSign
==========
*/
void R_DrawSign (msurface_t *signchain)
{
	extern int			sign_texture[2];
	msurface_t			*s;
	glpoly_t			*p;
	int					i;
	float				*v;
	static	float		coord;
	static	qboolean	sign0 = true;

	coord += host_frametime * 0.1;

	if (coord > 1)
	{
		coord -= 1;
		sign0 = !sign0;
	}

	if (sign0)
		glBindTexture (GL_TEXTURE_2D, sign_texture[0]);
	else
		glBindTexture (GL_TEXTURE_2D, sign_texture[1]);

	for (s = signchain; s; s=s->texturechain) 
	{
		for (p=s->polys ; p ; p=p->next)
		{
			glBegin (GL_POLYGON);
			for (i=0,v=p->verts[0] ; i<p->numverts ; i++, v+=VERTEXSIZE)
			{
				glTexCoord2f (v[3] + coord + 0.7, v[4]);
				glVertex3fv (v);
			}
			glEnd ();
		}
	}
}
