/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// quakedef.h -- primary header for client

#ifndef __QUAKEDEF__
#define __QUAKEDEF__

#define	VERSION	1.04

#define MAX_NUM_ARGVS	12

#define	MAX_QPATH		64			// max length of a miniracer game pathname
#define	MAX_OSPATH		128			// max length of a filesystem pathname

#define	MAX_MSGLEN		4096		// max length of a reliable message
#define	MAX_DATAGRAM	1024		// max length of unreliable message

//
// per-level limits
//

#define	MAX_EDICTS		512			// FIXME: ouch! ouch! ouch!
#define	MAX_MODELS		256			// these are sent over the net as bytes

//===========================================

#include <stdarg.h>

#ifndef _WIN32
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifndef _snprintf
#define _snprintf snprintf
#endif
#ifndef _vsnprintf
#define _vsnprintf vsnprintf
#endif
#endif

#include "common.h"
#include "bspfile.h"
#include "sys.h"
#include "zone.h"
#include "mathlib.h"

#include "draw.h"
#include "cvar.h"
#include "crc.h"
#include "screen.h"
#include "net.h"
#include "protocol.h"
#include "cmd.h"
#include "render.h"
#include "client.h"
#include "progs.h"
#include "server.h"

#include "gl_model.h"

#include "world.h"
#include "keys.h"
#include "console.h"

#include "glminiracer.h"

//=============================================================================

// the host system specifies the base of the directory tree, the
// command line parms passed to the program, and the amount of memory
// available for the program to use

typedef struct
{
	int		argc;
	char	**argv;
	void	*membase;
	int	memsize;
} MR_parms_t;

// mode_t is a system definition on Unix and has nothing in common with MR_mode_t !!!
typedef struct 
{
	int			width;
	int			height;
	int			bpp;
	qboolean	fullscreen;
} MR_mode_t;

//=============================================================================

//
// host
//
extern	MR_mode_t		mode;

extern	MR_parms_t	host_parms;

extern	cvar_t		developer;

extern	int			host_framecount;
extern	double		realtime;
extern	double		host_frametime;
extern	qboolean	isDedicated;
extern	qboolean	host_initialized;

void Host_ClearMemory (void);
void Host_ServerFrame (void);
void Host_InitCommands (void);
void Host_Init (MR_parms_t *parms);
void Host_Shutdown(void);
void Host_Error (char *error, ...);
void Host_EndGame (char *message, ...);
void Host_Frame (float time);
void Host_Quit_f (void);
void Host_ClientCommands (char *fmt, ...);
void Host_ShutdownServer (qboolean crash);

#endif //__QUAKEDEF__

