/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

// refresh.h -- public interface to refresh functions

//=============================================================================

typedef struct entity_state_s
{
	vec3_t	origin;
	vec3_t	angles;
	int		modelindex;
} entity_state_t;

typedef struct entity_s
{
	entity_state_t	baseline;		// to fill in defaults in updates
	struct model_s	*model;			// NULL = no model

	double			msgtime;		// time of last update
	double			time_sound;

	vec3_t			rear_skid[2];
	vec3_t			front_skid[2];
	vec3_t			origin;
	vec3_t			angles;	
	vec3_t			msg_origin[2];
	vec3_t			msg_angles[2];
	vec3_t			ground_normal;
	vec3_t			ground_pos;

	int				frame;
	int				visframe;		// last frame this entity was
									//  found in an active leaf
	float			spinangle;

	qboolean		forcelink;		// model changed
	qboolean		inair;
} entity_t;

typedef struct
{
	float		origin[3];
	float		angles[3];

	float		old_origin[3];
	float		old_angles[3];

	float		dest_origin[3];
	float		dest_angles[3];
} camera_t;

extern	vec3_t	vforward;
extern	vec3_t	vright;
extern	vec3_t	vup;

void R_Init (void);
void R_RenderView (void);		// must set camera first

void R_NewMap (void);
