/*
Copyright (C) 1996-1997 Id Software, Inc.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
// sv_main.c -- server main program

#include	"../quakedef.h"

server_t		sv;
server_static_t	svs;

char	localmodels[MAX_MODELS][5];

//============================================================================

/*
=======
SV_Init
=======
*/
void SV_Init (void)
{
	int		i;

	for (i=0 ; i<MAX_MODELS ; i++)
		_snprintf (localmodels[i], sizeof(localmodels[i]), "*%i", i);
}

/*
==============================================================================

CLIENT SPAWNING

==============================================================================
*/

/*
=================
SV_SendServerinfo
=================
*/
void SV_SendServerinfo (client_t *client)
{
	char	**s;
	char	*message;

	MSG_WriteByte  (&client->message, svc_serverinfo);
	MSG_WriteByte  (&client->message, svs.maxclients);

	message = sv.edicts->v.message;

	MSG_WriteString (&client->message,message);

	for (s = sv.model_precache+1 ; *s ; s++)
		MSG_WriteString (&client->message, *s);
	MSG_WriteByte  (&client->message, 0);

	MSG_WriteByte  (&client->message, svc_setview);
	MSG_WriteShort (&client->message, NUM_FOR_EDICT(client->edict));

	MSG_WriteByte  (&client->message, svc_signonnum);
	MSG_WriteByte  (&client->message, 1);

	client->sendsignon	= true;
	client->spawned		= false;
}

/*
================
SV_ConnectClient
================
*/
void SV_ConnectClient (int clientnum)
{
	edict_t			*ent;
	client_t		*client;
	int				edictnum;
	struct qsocket_s *netconnection;

	client = svs.clients + clientnum;

	Con_DPrintf ("Client %s connected\n", client->netconnection->address);

	edictnum = clientnum+1;

	ent = EDICT_NUM(edictnum);

	netconnection = client->netconnection;
	
	MR_memset (client, 0, sizeof(*client));
	client->netconnection = netconnection;

	MR_strcpy (client->name, "unconnected");
	client->active = true;
	client->spawned = false;
	client->edict = ent;
	client->message.data = client->msgbuf;
	client->message.maxsize = sizeof(client->msgbuf);
	client->message.allowoverflow = true;		// we can catch it

	SV_SendServerinfo (client);
}

/*
=====================
SV_CheckForNewClients
=====================
*/
void SV_CheckForNewClients (void)
{
	struct qsocket_s	*ret;
	int					i;

	while (1)
	{
		ret = NET_CheckNewConnections ();
		if (!ret)
			break;

		for (i=0 ; i<svs.maxclients ; i++)
			if (!svs.clients[i].active)
				break;
		if (i == svs.maxclients)
			Sys_Error ("Host_CheckForNewClients: no free clients");
		
		svs.clients[i].netconnection = ret;
		SV_ConnectClient (i);	
	
		net_activeconnections++;
	}
}

/*
===============================================================================

FRAME UPDATES

===============================================================================
*/

/*
================
SV_ClearDatagram
================
*/
void SV_ClearDatagram (void)
{
	SZ_Clear (&sv.datagram);
}

/*
========================
SV_WriteEntitiesToClient
========================
*/
void SV_WriteEntitiesToClient (edict_t *client, sizebuf_t *msg)
{
	int		e;
	int		bits;
	edict_t	*ent;

	ent = NEXT_EDICT(sv.edicts);

	for (e=1 ; e<sv.num_edicts ; e++, ent = NEXT_EDICT(ent))
	{
		if (ent != client && (!ent->v.modelindex || !ent->v.model[0]))
			continue;

		if (msg->maxsize - msg->cursize < 16)
		{
			Con_Printf ("packet overflow\n");
			return;
		}

// send an update
		bits = 0;

		if (ent->v.modelindex	!= ent->baseline.modelindex)	bits |= U_MODEL;
		if (ent->v.angles[0]	!= ent->baseline.angles[0])		bits |= U_ANGLE0;
		if (ent->v.angles[1]	!= ent->baseline.angles[1])		bits |= U_ANGLE1;
		if (ent->v.angles[2]	!= ent->baseline.angles[2])		bits |= U_ANGLE2;
		if (ent->game.origin[0]	!= ent->baseline.origin[0])		bits |= U_ORIGIN0;
		if (ent->game.origin[1]	!= ent->baseline.origin[1])		bits |= U_ORIGIN1;
		if (ent->game.origin[2]	!= ent->baseline.origin[2])		bits |= U_ORIGIN2;

//
// write the message
//
		MSG_WriteByte (msg,bits | U_SIGNAL);
		
		MSG_WriteByte (msg, e);

		if (bits & U_MODEL)		MSG_WriteByte	(msg, ent->v.modelindex);
		if (bits & U_ANGLE0)	MSG_WriteAngle	(msg, ent->v.angles[0]);
		if (bits & U_ANGLE1)	MSG_WriteAngle	(msg, ent->v.angles[1]);
		if (bits & U_ANGLE2)	MSG_WriteAngle	(msg, ent->v.angles[2]);
		if (bits & U_ORIGIN0)	MSG_WriteCoord	(msg, ent->game.origin[0]);
		if (bits & U_ORIGIN1)	MSG_WriteCoord	(msg, ent->game.origin[1]);
		if (bits & U_ORIGIN2)	MSG_WriteCoord	(msg, ent->game.origin[2]);
	}
}

/*
=====================
SV_SendClientDatagram
=====================
*/
qboolean SV_SendClientDatagram (client_t *client)
{
	byte		buf[MAX_DATAGRAM];
	sizebuf_t	msg;
	
	msg.data = buf;
	msg.maxsize = sizeof(buf);
	msg.cursize = 0;

	MSG_WriteByte (&msg, svc_time);
	MSG_WriteFloat (&msg, sv.time);

	SV_WriteEntitiesToClient (client->edict, &msg);

// copy the server datagram if there is space
	if (msg.cursize + sv.datagram.cursize < msg.maxsize)
		SZ_Write (&msg, sv.datagram.data, sv.datagram.cursize);

// send the datagram
	if (NET_SendUnreliableMessage (client->netconnection, &msg) == -1)
	{
		SV_DropClient (true);// if the message couldn't send, kick off
		return false;
	}
	
	return true;
}

/*
=======================
SV_UpdateToReliableMessages
=======================
*/
void SV_UpdateToReliableMessages (void)
{
	int			j;
	client_t	*client;

	for (j=0, client = svs.clients ; j<svs.maxclients ; j++, client++)
	{
		if (!client->active)
			continue;
		SZ_Write (&client->message, sv.reliable_datagram.data, sv.reliable_datagram.cursize);
	}

	SZ_Clear (&sv.reliable_datagram);
}

/*
==========
SV_SendNop
==========
*/
void SV_SendNop (client_t *client)
{
	sizebuf_t	msg;
	byte		buf[4];
	
	msg.data = buf;
	msg.maxsize = sizeof(buf);
	msg.cursize = 0;

	MSG_WriteByte (&msg, svc_nop);

	if (NET_SendUnreliableMessage (client->netconnection, &msg) == -1)
		SV_DropClient (true);	// if the message couldn't send, kick off
	client->last_message = realtime;
}

/*
=====================
SV_SendClientMessages
=====================
*/
void SV_SendClientMessages (void)
{
	int			i;
	
// update frags, names, etc
	SV_UpdateToReliableMessages ();

// build individual updates
	for (i=0, host_client = svs.clients ; i<svs.maxclients ; i++, host_client++)
	{
		if (!host_client->active)
			continue;

		if (host_client->spawned)
		{
			if (!SV_SendClientDatagram (host_client))
				continue;
		}
		else
		{
		// the player isn't totally in the game yet
		// send small keepalive messages if too much time has passed
		// send a full message when the next signon stage has been requested
		// some other message data (name changes, etc) may accumulate 
		// between signon stages
			if (!host_client->sendsignon)
			{
				if (realtime - host_client->last_message > 5)
					SV_SendNop (host_client);
				continue;	// don't send out non-signon messages
			}
		}

		// check for an overflowed message.  Should only happen
		// on a very fucked up connection that backs up a lot, then
		// changes level
		if (host_client->message.overflowed)
		{
			SV_DropClient (true);
			host_client->message.overflowed = false;
			continue;
		}
			
		if (host_client->message.cursize || host_client->dropasap)
		{
			if (!NET_CanSendMessage (host_client->netconnection))
			{
				continue;
			}

			if (host_client->dropasap)
				SV_DropClient (false);	// went to another level
			else
			{
				if (NET_SendMessage (host_client->netconnection, &host_client->message) == -1)
					SV_DropClient (true);	// if the message couldn't send, kick off
				SZ_Clear (&host_client->message);
				host_client->last_message = realtime;
				host_client->sendsignon = false;
			}
		}
	}
}

/*
===============
SERVER SPAWNING
===============
*/

/*
=============
SV_ModelIndex
=============
*/
int SV_ModelIndex (char *name)
{
	int		i;
	
	if (!name || !name[0])
		return 0;

	for (i=0 ; i<MAX_MODELS && sv.model_precache[i] ; i++)
		if (!MR_strcmp (sv.model_precache[i], name))
			return i;
	if (i==MAX_MODELS || !sv.model_precache[i])
		Sys_Error ("SV_ModelIndex: model %s not precached", name);
	return i;
}

/*
================
SV_CreateBaseline
================
*/
void SV_CreateBaseline (void)
{
	edict_t			*svent;
	int				entnum;	
		
	for (entnum = 0; entnum < sv.num_edicts ; entnum++)
	{
	// get the current server version
		svent = EDICT_NUM(entnum);
		if (svent->free)
			continue;
		if (entnum > svs.maxclients && !svent->v.modelindex)
			continue;

	//
	// create entity baseline
	//
		VectorCopy (svent->game.origin, svent->baseline.origin);
		VectorCopy (svent->v.angles, svent->baseline.angles);

		if (entnum > 0 && entnum <= svs.maxclients)
			svent->baseline.modelindex	= SV_ModelIndex ("models/F40.car");
		else
			svent->baseline.modelindex	= SV_ModelIndex (svent->v.model);
		
	//
	// add to the message
	//
		MSG_WriteByte (&sv.signon,svc_spawnbaseline);		
		MSG_WriteShort (&sv.signon,entnum);

		MSG_WriteByte	(&sv.signon, svent->baseline.modelindex);
		MSG_WriteAngle	(&sv.signon, svent->baseline.angles[0]);
		MSG_WriteAngle	(&sv.signon, svent->baseline.angles[1]);
		MSG_WriteAngle	(&sv.signon, svent->baseline.angles[2]);
		MSG_WriteCoord	(&sv.signon, svent->baseline.origin[0]);
		MSG_WriteCoord	(&sv.signon, svent->baseline.origin[1]);
		MSG_WriteCoord	(&sv.signon, svent->baseline.origin[2]);
	}
}

/*
================
SV_SendReconnect

Tell all the clients that the server is changing levels
================
*/
void SV_SendReconnect (void)
{
	char	data[128];
	sizebuf_t	msg;

	msg.data = data;
	msg.cursize = 0;
	msg.maxsize = sizeof(data);

	MSG_WriteByte (&msg, svc_stufftext);
	MSG_WriteString (&msg, "reconnect\n");
	NET_SendToAll (&msg, 5);
	
	if (cls.state != ca_dedicated)
		Cmd_ExecuteString ("reconnect\n", src_command);
}

/*
==============
SV_SpawnServer

This is called at the start of each level
==============
*/
void SV_SpawnServer (char *server)
{
	edict_t		*ent;
	int			i;

	// let's not have any servers with no name
	if (hostname.string[0] == 0)
		Cvar_Set ("hostname", "UNNAMED");

	Con_DPrintf ("SpawnServer: %s\n",server);
	svs.changelevel_issued = false;		// now safe to issue another

//
// tell all connected clients that we are going to a new level
//
	if (sv.active)
		SV_SendReconnect ();

//
// set up the new server
//
	Host_ClearMemory ();

	MR_memset (&sv, 0, sizeof(sv));

	MR_strcpy (sv.name, server);

// allocate server memory
	sv.max_edicts = MAX_EDICTS;
	
	sv.edicts = Hunk_AllocName (sv.max_edicts*sizeof(edict_t), "edicts");
	sv.edicts->game.touch = 0;

	sv.datagram.maxsize = sizeof(sv.datagram_buf);
	sv.datagram.cursize = 0;
	sv.datagram.data = sv.datagram_buf;
	
	sv.reliable_datagram.maxsize = sizeof(sv.reliable_datagram_buf);
	sv.reliable_datagram.cursize = 0;
	sv.reliable_datagram.data = sv.reliable_datagram_buf;
	
	sv.signon.maxsize = sizeof(sv.signon_buf);
	sv.signon.cursize = 0;
	sv.signon.data = sv.signon_buf;
	
// leave slots at start for clients only
	sv.num_edicts = svs.maxclients+1;
	for (i=0 ; i<svs.maxclients ; i++)
	{
		ent = EDICT_NUM(i+1);
		svs.clients[i].edict = ent;
	}
	
	sv.state = ss_loading;

	sv.time = 1.0;
	
	MR_strcpy (sv.name, server);
	_snprintf (sv.modelname, sizeof(sv.modelname), "maps/%s.bsp", server);
	sv.worldmodel = Mod_ForName (sv.modelname, false);
	if (!sv.worldmodel)
	{
		Con_Printf ("Couldn't spawn server %s\n", sv.modelname);
		sv.active = false;
		return;
	}
	sv.models[1] = sv.worldmodel;
	
//
// clear world interaction links
//
	SV_ClearWorld ();
	
	sv.model_precache[0] = "";
	sv.model_precache[1] = sv.modelname;
	for (i=1 ; i<sv.worldmodel->numsubmodels ; i++)
	{
		sv.model_precache[1+i] = localmodels[i];
		sv.models[i+1] = Mod_ForName (localmodels[i], false);
	}

//
// load the rest of the entities
//	
	ent = EDICT_NUM(0);
	MR_memset (&ent->v,    0, sizeof(ent->v));
	MR_memset (&ent->game, 0, sizeof(ent->game));

	ent->free			= false;
	ent->v.model		= sv.worldmodel->name;
	ent->v.modelindex	= 1;		// world model
	ent->game.solid		= SOLID_BSP;
	ent->game.movetype	= MOVETYPE_PUSH;
	
	miniracer.world		= ent;

	ED_LoadFromFile (sv.worldmodel->entities);

	for(i=0; i<miniracer.TotalWays; i++)
	{
		if (miniracer.waypoint[i].used == false)
			Host_Error ("Missing waypoint: %i", i);
	}

	for(i=0; i<8; i++)
	{
		if (miniracer.startpoint[i].used == false)
			Host_Error ("Missing startpoint: %i", i);

		if (miniracer.finishpoint[i].used == false)
			Host_Error ("Missing finishpoint: %i", i);
	}

	sv.active = true;

// all setup is completed, any further precache statements are errors
	sv.state = ss_active;
	
// run two frames to allow everything to settle
	host_frametime = 0.1;
	SV_Physics ();
	SV_Physics ();

// create a baseline for more efficient communications
	SV_CreateBaseline ();

// send serverinfo to all connected clients
	for (i=0,host_client = svs.clients ; i<svs.maxclients ; i++, host_client++)
		if (host_client->active)
			SV_SendServerinfo (host_client);
	
	Con_DPrintf ("Server spawned.\n");
}
